from .types import t_scheduler
from .manager import Manager
import pretty_cron
import requests
import pycron


class Scheduler:
    naas = None
    role = t_scheduler

    def __init__(self):
        self.manager = Manager()
        self.get_output = self.manager.get_output
        self.clear_output = self.manager.clear_output
        self.list = self.manager.list_prod
        self.clear = self.manager.clear_prod
        self.get = self.manager.get_prod

    def status(self):
        req = requests.get(url=f"{self.manager.naas_api}/scheduler")
        req.raise_for_status()
        jsn = req.json()
        print(jsn)
        return jsn

    def pause(self):
        req = requests.get(url=f"{self.manager.naas_api}/scheduler/pause")
        req.raise_for_status()
        jsn = req.json()
        print(jsn)
        return jsn

    def resume(self):
        req = requests.get(url=f"{self.manager.naas_api}/scheduler/resume")
        req.raise_for_status()
        jsn = req.json()
        print(jsn)
        return jsn

    def currents(self, raw=False):
        json_data = self.manager.get_naas()
        if raw:
            for item in json_data:
                if item["type"] == self.role:
                    print(item)
        else:
            for item in json_data:
                kind = None
                if item["type"] == self.role:
                    cron_string = pretty_cron.prettify_cron(item["value"])
                    kind = f"scheduler {cron_string}"
                    print(f"File ==> {item['path']} is {kind}")

    def __check_cron(self, text):
        res = False
        try:
            pycron.is_now(text)
            res = True
        except ValueError:
            pass
        return res

    def add(self, path=None, recurrence=None, params=None, debug=False):
        if self.manager.is_production():
            print("No add done you are in production\n")
            return
        if not recurrence:
            print("No recurrence provided\n")
            return
        if not self.__check_cron(recurrence):
            print(f"WARNING : Recurrence wrong format {recurrence}")
            return
        current_file = self.manager.get_path(path)
        self.manager.add_prod(
            {
                "type": self.role,
                "path": current_file,
                "params": {},
                "value": recurrence,
            },
            debug,
        )
        cron_string = pretty_cron.prettify_cron(recurrence)
        print("👌 Well done! Your Notebook has been sent to production. \n")
        print(
            f'⏰ It will be scheduled "{cron_string}" (more on the syntax on https://crontab.guru/).\n'
        )
        print('Ps: to remove the "Scheduler", just replace .add by .delete')

    def delete(self, path=None, all=False, debug=False):
        if self.manager.is_production():
            print("No delete done you are in production\n")
            return
        current_file = self.manager.get_path(path)
        self.manager.del_prod({"type": self.role, "path": current_file}, debug)
        print("🗑 Done! Your Scheduler has been remove from production.\n")
        if all is True:
            self.clear(path)
            self.clear_output(path)



from base64 import b64encode
import escapism
import string
import os
import requests

_docker_safe_chars = set(string.ascii_letters + string.digits)
_docker_escape_char_kubernet = "-"
_docker_escape_char_docker = "_"


def escape_kubernet(s):
    """Escape a string to kubernet-safe characters"""
    return escapism.escape(
        s,
        safe=_docker_safe_chars,
        escape_char=_docker_escape_char_kubernet,
    )


def escape_docker(s):
    """Escape a string to docker-safe characters"""
    return escapism.escape(
        s,
        safe=_docker_safe_chars,
        escape_char=_docker_escape_char_docker,
    )


def encode_proxy_url(token=""):
    client = os.environ.get("JUPYTERHUB_USER", "")
    base_public_url = os.environ.get("PUBLIC_PROXY_API", "")
    client_encoded = escape_kubernet(client)
    message_bytes = client_encoded.encode("ascii")
    base64_bytes = b64encode(message_bytes)
    username_base64 = base64_bytes.decode("ascii")
    return f"{base_public_url}/{username_base64}/{token}"


class Domain:

    base_notif_url = os.environ.get("PUBLIC_PROXY_API", None)
    headers = None

    def __init__(self):
        self.headers = {
            "Authorization": f'token {os.environ.get("JUPYTERHUB_API_TOKEN", None)}'
        }

    def status(self):
        req = requests.get(url=f"{self.base_notif_url}/status")
        req.raise_for_status()
        jsn = req.json()
        return jsn

    def add(self, domain, url=None):
        token = None
        endpoint = None
        if url:
            list_url = url.split("/")
            token = list_url.pop()
            endpoint = list_url.pop()
        if "://" in domain:
            clean_domain = domain.split("://")[1]
        else:
            clean_domain = domain
        data = {"domain": clean_domain, "endpoint": endpoint, "token": token}
        req = requests.post(
            url=f"{self.base_notif_url}/proxy", headers=self.headers, json=data
        )
        req.raise_for_status()
        new_url = f"https://{clean_domain}"
        if token:
            new_url = f"{new_url}/{endpoint}/{token}"
        return new_url

    def get(self, domain):
        req = requests.get(
            url=f"{self.base_notif_url}/proxy",
            headers=self.headers,
            json={"domain": domain},
        )
        req.raise_for_status()
        jsn = req.json()
        return jsn

    def delete(self, domain):
        req = requests.delete(
            url=f"{self.base_notif_url}/proxy",
            headers=self.headers,
            json={"domain": domain},
        )
        req.raise_for_status()
        jsn = req.json()
        return jsn



from .types import t_dependency
from .manager import Manager


class Dependency:
    naas = None
    role = t_dependency

    def __init__(self):
        self.manager = Manager()
        self.get = self.manager.get_prod
        self.list = self.manager.list_prod
        self.clear = self.manager.clear_prod

    def currents(self, raw=False):
        json_data = self.manager.get_naas()
        if raw:
            for item in json_data:
                if item["type"] == self.role:
                    print(item)
        else:
            for item in json_data:
                if item["type"] == self.role:
                    print(f"File ==> {item['path']}")

    def add(self, path=None, debug=False):
        if self.manager.is_production():
            print("No add done you are in production\n")
            return
        current_file = self.manager.get_path(path)
        self.manager.add_prod(
            {
                "type": self.role,
                "path": current_file,
                "params": {},
                "value": "Only internal",
            },
            debug,
        )
        print("👌 Well done! Your Dependency has been sent to production. \n")
        print('PS: to remove the "Dependency" feature, just replace .add by .delete')

    def delete(self, path=None, all=False, debug=False):
        if self.manager.is_production():
            print("No delete done you are in production\n")
            return
        current_file = self.manager.get_path(path)
        self.manager.del_prod({"type": self.role, "path": current_file}, debug)
        print("🗑 Done! Your Dependency has been remove from production.\n")
        if all is True:
            self.clear(path)
